/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "inISearchProcess.idl"

interface nsIDOMDocument;

[scriptable, uuid(e0d39e48-1dd1-11b2-81bd-9a0c117f0736)]
interface inICSSValueSearch : inISearchProcess {
  
  attribute nsIDOMDocument document;
  
  // the base url for all returned URL results, if returnRelativeURLs is true
  attribute wstring baseURL; 
  
  // strip off the baseURL for all URL results if true
  attribute boolean returnRelativeURLs;

  // correct the paths on a chrome url, such as turning global/skin/blah into global/blah
  attribute boolean normalizeChromeURLs;

  // add a css property to search for
  void addPropertyCriteria(in wstring aPropName);
  
  // set the text value to search for in the properties specified (optional)
  attribute wstring textCriteria;

};
